@@ -147,13 +147,15 @@ class ScopeValidationTestCase(TestCase):
         mock_scope2.name = "esi-characters.write_contacts.v1"
 
         mock_token = Mock()
-        mock_token.scopes.values_list.return_value = [
-            "esi-characters.read_contacts.v1",
-            "esi-characters.write_contacts.v1",
-        ]
+        mock_token.scopes.all.return_value = [mock_scope1, mock_scope2]
 
+        # Mock the queryset chain: filter().require_valid().prefetch_related() -> list of tokens
         mock_token_qs = Mock()
-        mock_token_qs.require_valid.return_value.first.return_value = mock_token
+        mock_prefetch_qs = Mock()
+        mock_prefetch_qs.__iter__ = Mock(return_value=iter([mock_token]))
+        mock_token_qs.require_valid.return_value.prefetch_related.return_value = (
+            mock_prefetch_qs
+        )
         mock_token_filter.return_value = mock_token_qs
 
         has_scopes, missing = character_has_required_scopes(character, self.user)
@@ -172,13 +174,19 @@ class ScopeValidationTestCase(TestCase):
         )
 
         # Mock token with only one scope
-        mock_token = Mock()
-        mock_token.scopes.values_list.return_value = [
-            "esi-characters.read_contacts.v1",
-        ]
+        mock_scope1 = Mock()
+        mock_scope1.name = "esi-characters.read_contacts.v1"
 
+        mock_token = Mock()
+        mock_token.scopes.all.return_value = [mock_scope1]
+
+        # Mock the queryset chain: filter().require_valid().prefetch_related() -> list of tokens
         mock_token_qs = Mock()
-        mock_token_qs.require_valid.return_value.first.return_value = mock_token
+        mock_prefetch_qs = Mock()
+        mock_prefetch_qs.__iter__ = Mock(return_value=iter([mock_token]))
+        mock_token_qs.require_valid.return_value.prefetch_related.return_value = (
+            mock_prefetch_qs
+        )
         mock_token_filter.return_value = mock_token_qs
 
         has_scopes, missing = character_has_required_scopes(character, self.user)
@@ -196,9 +204,13 @@ class ScopeValidationTestCase(TestCase):
             corporation_name="Test Corp",
         )
 
-        # Mock no token
+        # Mock no token - return empty list
         mock_token_qs = Mock()
-        mock_token_qs.require_valid.return_value.first.return_value = None
+        mock_prefetch_qs = Mock()
+        mock_prefetch_qs.__iter__ = Mock(return_value=iter([]))
+        mock_token_qs.require_valid.return_value.prefetch_related.return_value = (
+            mock_prefetch_qs
+        )
         mock_token_filter.return_value = mock_token_qs
 
         has_scopes, missing = character_has_required_scopes(character, self.user)
@@ -237,19 +249,25 @@ class CorporationTokenValidationTestCase(TestCase):
 
         self.assertIn("no characters", str(context.exception).lower())
 
-    @patch("standingsmanager.validators.character_has_required_scopes")
+    @patch("standingsmanager.validators.get_required_scopes_for_user")
     def test_validate_corporation_token_coverage_all_tokens_valid(
-        self, mock_has_scopes
+        self, mock_get_scopes
     ):
         """Test validation passes when all characters have valid tokens."""
+        from esi.models import Scope, Token
+
         # Use a different corporation for this test to avoid conflicts
+        # Set member_count to 2 to match the characters we'll create
         test_corp, _ = EveCorporationInfo.objects.get_or_create(
             corporation_id=98000002,
             defaults={
                 "corporation_name": "Test Corporation 2",
-                "member_count": 100,
+                "member_count": 2,
             },
         )
+        # Update member count in case it already existed with different value
+        test_corp.member_count = 2
+        test_corp.save()
 
         # Create characters in corp
         char1 = EveCharacter.objects.create(
@@ -273,8 +291,32 @@ class CorporationTokenValidationTestCase(TestCase):
             user=self.user, character=char2, owner_hash="hash2"
         )
 
-        # Mock all characters have scopes
-        mock_has_scopes.return_value = (True, [])
+        # Mock required scopes
+        mock_get_scopes.return_value = ["esi-characters.read_contacts.v1"]
+
+        # Create scope
+        scope, _ = Scope.objects.get_or_create(name="esi-characters.read_contacts.v1")
+
+        # Create valid tokens for both characters
+        token1 = Token.objects.create(
+            user=self.user,
+            character_id=char1.character_id,
+            character_name=char1.character_name,
+            character_owner_hash="hash1",
+            access_token="test_access_token_1",
+            refresh_token="test_refresh_token_1",
+        )
+        token1.scopes.add(scope)
+
+        token2 = Token.objects.create(
+            user=self.user,
+            character_id=char2.character_id,
+            character_name=char2.character_name,
+            character_owner_hash="hash2",
+            access_token="test_access_token_2",
+            refresh_token="test_refresh_token_2",
+        )
+        token2.scopes.add(scope)
 
         has_coverage, missing_chars = validate_corporation_token_coverage(
             test_corp, self.user
@@ -283,17 +325,23 @@ class CorporationTokenValidationTestCase(TestCase):
         self.assertTrue(has_coverage)
         self.assertEqual(missing_chars, [])
 
-    @patch("standingsmanager.validators.character_has_required_scopes")
-    def test_validate_corporation_token_coverage_missing_tokens(self, mock_has_scopes):
+    @patch("standingsmanager.validators.get_required_scopes_for_user")
+    def test_validate_corporation_token_coverage_missing_tokens(self, mock_get_scopes):
         """Test validation fails when some characters lack tokens."""
+        from esi.models import Scope, Token
+
         # Use a different corporation for this test to avoid conflicts
+        # Set member_count to 2 to match the characters we'll create
         test_corp, _ = EveCorporationInfo.objects.get_or_create(
             corporation_id=98000003,
             defaults={
                 "corporation_name": "Test Corporation 3",
-                "member_count": 100,
+                "member_count": 2,
             },
         )
+        # Update member count in case it already existed with different value
+        test_corp.member_count = 2
+        test_corp.save()
 
         # Create characters in corp with unique IDs
         char1 = EveCharacter.objects.create(
@@ -311,20 +359,28 @@ class CorporationTokenValidationTestCase(TestCase):
 
         # Create ownership
         CharacterOwnership.objects.create(
-            user=self.user, character=char1, owner_hash="hash1"
+            user=self.user, character=char1, owner_hash="hash3"
         )
         CharacterOwnership.objects.create(
-            user=self.user, character=char2, owner_hash="hash2"
+            user=self.user, character=char2, owner_hash="hash4"
         )
 
-        # Mock one character missing scopes
-        def mock_scopes_side_effect(char, user):
-            if char.character_id == 12347:
-                return (True, [])
-            else:
-                return (False, ["esi-characters.write_contacts.v1"])
+        # Mock required scopes
+        mock_get_scopes.return_value = ["esi-characters.read_contacts.v1"]
 
-        mock_has_scopes.side_effect = mock_scopes_side_effect
+        # Create scope
+        scope, _ = Scope.objects.get_or_create(name="esi-characters.read_contacts.v1")
+
+        # Create valid token only for char1 (char2 has no token)
+        token1 = Token.objects.create(
+            user=self.user,
+            character_id=char1.character_id,
+            character_name=char1.character_name,
+            character_owner_hash="hash3",
+            access_token="test_access_token_3",
+            refresh_token="test_refresh_token_3",
+        )
+        token1.scopes.add(scope)
 
         has_coverage, missing_chars = validate_corporation_token_coverage(
             test_corp, self.user
