@@ -1,11 +1,13 @@
 """Views for standingsmanager."""
 
 import csv
+import json
 
 from django.contrib.auth.decorators import login_required, permission_required
+from django.core.exceptions import ValidationError
+from django.db import IntegrityError
 from django.http import HttpResponse, JsonResponse
 from django.shortcuts import get_object_or_404, redirect, render
-from django.urls import reverse
 from django.utils import timezone
 from django.views.decorators.http import require_http_methods
 from esi.decorators import token_required
@@ -27,7 +29,6 @@ from .models import (
 from .validators import (
     can_user_request_character_standing,
     can_user_request_corporation_standing,
-    character_has_required_scopes,
     get_required_scopes_for_user,
     validate_corporation_token_coverage,
 )
@@ -60,18 +61,24 @@ def index(request):
 
 @login_required
 @permission_required("standingsmanager.add_syncedcharacter")
-@token_required(scopes=SyncedCharacter.get_esi_scopes())
+@token_required(scopes=SyncedCharacter.get_esi_scopes(), new=True)
 def add_scopes(request, token):
     """
     View that triggers EVE OAuth with required scopes.
 
-    Uses token_required decorator which will:
-    1. Redirect to EVE SSO if no valid token with scopes exists
+    Uses token_required decorator with new=True which will:
+    1. Always redirect to EVE SSO to get a fresh token with required scopes
     2. After successful auth, redirect back here with the new token
     3. Then redirect to the request standings page
 
     The token parameter is provided by the decorator after successful auth.
     """
+    # Log token info for debugging
+    logger.info(
+        f"Token added for character_id={token.character_id}, "
+        f"character_name={token.character_name}, "
+        f"scopes={list(token.scopes.values_list('name', flat=True))}"
+    )
     # Token was successfully added/updated with required scopes
     # Redirect back to the request standings page
     return redirect("standingsmanager:request_standings")
@@ -107,55 +114,93 @@ def request_standings(request):
     # Get required scopes for this user
     required_scopes = get_required_scopes_for_user(user)
 
+    # Pre-fetch all character IDs for bulk queries
+    character_ids = [co.character.character_id for co in character_ownerships]
+
+    # Bulk fetch all entities for these characters
+    character_entities = {
+        e.id: e
+        for e in EveEntity.objects.filter(
+            id__in=character_ids, category=EveEntity.CATEGORY_CHARACTER
+        )
+    }
+
+    # Bulk fetch standings for all character entities
+    entity_ids_with_standings = set(
+        StandingsEntry.objects.filter(
+            eve_entity_id__in=character_entities.keys()
+        ).values_list("eve_entity_id", flat=True)
+    )
+
+    # Bulk fetch pending requests for all character entities
+    entity_ids_with_pending = set(
+        StandingRequest.objects.filter(
+            eve_entity_id__in=character_entities.keys(),
+            state=StandingRequest.State.PENDING,
+        ).values_list("eve_entity_id", flat=True)
+    )
+
+    # Bulk fetch all valid tokens for this user with scopes
+    from esi.models import Token
+
+    user_tokens = list(
+        Token.objects.filter(user=user, character_id__in=character_ids)
+        .require_valid()
+        .prefetch_related("scopes")
+    )
+
+    # Build a map of character_id -> set of scopes
+    character_scopes = {}
+    for token in user_tokens:
+        if token.character_id not in character_scopes:
+            character_scopes[token.character_id] = set()
+        character_scopes[token.character_id].update(
+            scope.name for scope in token.scopes.all()
+        )
+
+    required_scopes_set = set(required_scopes)
+
     for co in character_ownerships:
         character = co.character
         character_id = character.character_id
 
-        # Check if character has standing
-        try:
-            entity = EveEntity.objects.get(
-                id=character_id, category=EveEntity.CATEGORY_CHARACTER
-            )
-            has_standing = StandingsEntry.objects.filter(eve_entity=entity).exists()
-        except EveEntity.DoesNotExist:
-            has_standing = False
+        # Check if character has standing (using pre-fetched data)
+        has_standing = character_id in entity_ids_with_standings
 
-        # Check if there's a pending request
-        try:
-            entity = EveEntity.objects.get(
-                id=character_id, category=EveEntity.CATEGORY_CHARACTER
-            )
-            pending_request = StandingRequest.objects.filter(
-                eve_entity=entity, state=StandingRequest.State.PENDING
-            ).exists()
-        except EveEntity.DoesNotExist:
-            pending_request = False
+        # Check if there's a pending request (using pre-fetched data)
+        pending_request = character_id in entity_ids_with_pending
 
-        # Check eligibility
-        can_request, error_message = can_user_request_character_standing(
-            character, user
-        )
+        # Check scopes (using pre-fetched data)
+        token_scopes = character_scopes.get(character_id, set())
+        missing_scopes = list(required_scopes_set - token_scopes)
+        has_scopes = len(missing_scopes) == 0
 
-        # Check scopes
-        has_scopes, missing_scopes = character_has_required_scopes(character, user)
-
-        # Determine status
+        # Determine eligibility and status
         if has_standing:
             status = "approved"
             status_text = "Approved"
+            can_request = False
             can_remove = True
+            error_message = None
         elif pending_request:
             status = "pending"
             status_text = "Pending Approval"
+            can_request = False
             can_remove = False
-        elif can_request:
+            error_message = None
+        elif not has_scopes:
+            status = "cannot_request"
+            scope_list = ", ".join(missing_scopes)
+            status_text = "Missing scopes"
+            can_request = False
+            can_remove = False
+            error_message = f"Missing required scopes: {scope_list}"
+        else:
             status = "can_request"
             status_text = "Can Request"
+            can_request = True
             can_remove = False
-        else:
-            status = "cannot_request"
-            status_text = error_message or "Cannot Request"
-            can_remove = False
+            error_message = None
 
         character_data = {
             "id": character_id,
@@ -193,64 +238,94 @@ def request_standings(request):
 
     # Process corporation data
     corporations_list = []
+
+    # Bulk fetch corporation entities
+    corp_ids = list(corporations_data.keys())
+    corp_entities = {
+        e.id: e
+        for e in EveEntity.objects.filter(
+            id__in=corp_ids, category=EveEntity.CATEGORY_CORPORATION
+        )
+    }
+
+    # Bulk fetch standings for corporations
+    corp_ids_with_standings = set(
+        StandingsEntry.objects.filter(
+            eve_entity_id__in=corp_entities.keys()
+        ).values_list("eve_entity_id", flat=True)
+    )
+
+    # Bulk fetch pending requests for corporations
+    corp_ids_with_pending = set(
+        StandingRequest.objects.filter(
+            eve_entity_id__in=corp_entities.keys(), state=StandingRequest.State.PENDING
+        ).values_list("eve_entity_id", flat=True)
+    )
+
+    # Bulk fetch EveCorporationInfo
+    corp_infos = {
+        c.corporation_id: c
+        for c in EveCorporationInfo.objects.filter(corporation_id__in=corp_ids)
+    }
+
     for corp_data in corporations_data.values():
         corp_id = corp_data["id"]
 
-        # Check if corporation has standing
-        try:
-            entity = EveEntity.objects.get(
-                id=corp_id, category=EveEntity.CATEGORY_CORPORATION
-            )
-            has_standing = StandingsEntry.objects.filter(eve_entity=entity).exists()
-        except EveEntity.DoesNotExist:
-            has_standing = False
+        # Check if corporation has standing (using pre-fetched data)
+        has_standing = corp_id in corp_ids_with_standings
 
-        # Check if there's a pending request
-        try:
-            entity = EveEntity.objects.get(
-                id=corp_id, category=EveEntity.CATEGORY_CORPORATION
-            )
-            pending_request = StandingRequest.objects.filter(
-                eve_entity=entity, state=StandingRequest.State.PENDING
-            ).exists()
-        except EveEntity.DoesNotExist:
-            pending_request = False
+        # Check if there's a pending request (using pre-fetched data)
+        pending_request = corp_id in corp_ids_with_pending
 
-        # Check token coverage
-        try:
-            corp_info = EveCorporationInfo.objects.get(corporation_id=corp_id)
-            has_full_coverage, missing_characters = validate_corporation_token_coverage(
-                corp_info, user
-            )
-        except EveCorporationInfo.DoesNotExist:
+        # Get or create corporation info
+        corp_info = corp_infos.get(corp_id)
+        if not corp_info:
             # Create corporation info if it doesn't exist
-            corp_info = EveCorporationInfo.objects.create_corporation(corp_id)
-            has_full_coverage, missing_characters = validate_corporation_token_coverage(
-                corp_info, user
-            )
+            # Handle race condition where another process may have created it
+            try:
+                corp_info = EveCorporationInfo.objects.create_corporation(corp_id)
+            except IntegrityError:
+                # Another process created it, fetch it
+                corp_info = EveCorporationInfo.objects.get(corporation_id=corp_id)
 
-        # Check eligibility
-        can_request, error_message = can_user_request_corporation_standing(
+        # Check token coverage for all corp members
+        has_full_coverage, missing_characters = validate_corporation_token_coverage(
             corp_info, user
         )
 
-        # Determine status
+        # Get actual member count from corp info
+        actual_member_count = corp_info.member_count
+        registered_count = EveCharacter.objects.filter(
+            corporation_id=corp_id,
+            character_ownership__isnull=False,
+        ).count()
+
+        # Determine eligibility and status
         if has_standing:
             status = "approved"
             status_text = "Approved"
+            can_request = False
             can_remove = True
+            error_message = None
         elif pending_request:
             status = "pending"
             status_text = "Pending Approval"
+            can_request = False
             can_remove = False
-        elif can_request:
+            error_message = None
+        elif not has_full_coverage:
+            status = "cannot_request"
+            char_list = ", ".join(missing_characters)
+            status_text = "Incomplete"
+            can_request = False
+            can_remove = False
+            error_message = f"Missing: {char_list}"
+        else:
             status = "can_request"
             status_text = "Can Request"
+            can_request = True
             can_remove = False
-        else:
-            status = "cannot_request"
-            status_text = error_message or "Cannot Request"
-            can_remove = False
+            error_message = None
 
         corp_data.update(
             {
@@ -261,6 +336,8 @@ def request_standings(request):
                 "has_full_coverage": has_full_coverage,
                 "missing_characters": missing_characters,
                 "error_message": error_message,
+                "actual_member_count": actual_member_count,
+                "registered_count": registered_count,
             }
         )
         corporations_list.append(corp_data)
@@ -456,9 +533,7 @@ def manage_revocations(request):
     """
     # Get all pending revocations
     pending_revocations = (
-        StandingRevocation.objects.filter(
-            state=StandingRevocation.RevocationState.PENDING
-        )
+        StandingRevocation.objects.filter(state=StandingRevocation.State.PENDING)
         .select_related("eve_entity", "requested_by")
         .order_by("-request_date")
     )
@@ -743,7 +818,16 @@ def api_request_corporation_standing(request, corporation_id):
         try:
             corporation = EveCorporationInfo.objects.get(corporation_id=corporation_id)
         except EveCorporationInfo.DoesNotExist:
-            corporation = EveCorporationInfo.objects.create_corporation(corporation_id)
+            # Handle race condition where another process may have created it
+            try:
+                corporation = EveCorporationInfo.objects.create_corporation(
+                    corporation_id
+                )
+            except IntegrityError:
+                # Another process created it, fetch it
+                corporation = EveCorporationInfo.objects.get(
+                    corporation_id=corporation_id
+                )
 
         # Check eligibility
         can_request, error_message = can_user_request_corporation_standing(
@@ -822,11 +906,27 @@ def api_remove_standing(request, entity_id):
                 )
 
         # Create revocation
-        revocation = StandingRevocation.objects.create_for_entity(
-            entity,
-            reason=StandingRevocation.RevocationReason.USER_REQUEST,
-            user=request.user,
-        )
+        try:
+            revocation = StandingRevocation.objects.create_for_entity(
+                entity,
+                reason=StandingRevocation.RevocationReason.USER_REQUEST,
+                user=request.user,
+            )
+        except ValidationError as e:
+            # Handle Django ValidationError message extraction
+            if hasattr(e, "messages") and e.messages:
+                error_msg = str(e.messages[0])
+            elif hasattr(e, "message"):
+                error_msg = str(e.message)
+            else:
+                error_msg = str(e)
+            logger.warning(
+                f"Validation error creating revocation for {entity.name}: {error_msg}"
+            )
+            return JsonResponse(
+                {"success": False, "error": error_msg},
+                status=400,
+            )
 
         logger.info(
             f"User {request.user.username} created revocation request for {entity.name}"
@@ -840,8 +940,295 @@ def api_remove_standing(request, entity_id):
             }
         )
 
+    except Exception:
+        logger.exception(f"Error creating revocation request for entity_id={entity_id}")
+        return JsonResponse(
+            {"success": False, "error": "An unexpected error occurred."}, status=500
+        )
+
+
+@login_required
+@permission_required("standingsmanager.add_syncedcharacter")
+@require_http_methods(["POST"])
+def api_bulk_request_character_standings(request):
+    """
+    API endpoint to create standing requests for multiple characters.
+
+    Expects JSON body with:
+        character_ids: list of EVE character IDs
+
+    Returns:
+        JSON response with success/error status and details for each character
+    """
+    try:
+        # Parse JSON body
+        try:
+            data = json.loads(request.body)
+            character_ids = data.get("character_ids", [])
+        except json.JSONDecodeError:
+            return JsonResponse(
+                {"success": False, "error": "Invalid JSON body."}, status=400
+            )
+
+        if not character_ids:
+            return JsonResponse(
+                {"success": False, "error": "No characters specified."}, status=400
+            )
+
+        results = []
+        success_count = 0
+        error_count = 0
+
+        for character_id in character_ids:
+            try:
+                # Get character
+                try:
+                    character = EveCharacter.objects.get(character_id=character_id)
+                except EveCharacter.DoesNotExist:
+                    results.append(
+                        {
+                            "character_id": character_id,
+                            "success": False,
+                            "error": "Character not found.",
+                        }
+                    )
+                    error_count += 1
+                    continue
+
+                # Check if user owns character
+                if not CharacterOwnership.objects.filter(
+                    user=request.user, character=character
+                ).exists():
+                    results.append(
+                        {
+                            "character_id": character_id,
+                            "character_name": character.character_name,
+                            "success": False,
+                            "error": "You do not own this character.",
+                        }
+                    )
+                    error_count += 1
+                    continue
+
+                # Check eligibility
+                can_request, error_message = can_user_request_character_standing(
+                    character, request.user
+                )
+                if not can_request:
+                    results.append(
+                        {
+                            "character_id": character_id,
+                            "character_name": character.character_name,
+                            "success": False,
+                            "error": error_message,
+                        }
+                    )
+                    error_count += 1
+                    continue
+
+                # Create request
+                standing_request = StandingRequest.objects.create_character_request(
+                    character, request.user
+                )
+
+                logger.info(
+                    f"User {request.user.username} created standing request "
+                    f"for character {character.character_name}"
+                )
+
+                results.append(
+                    {
+                        "character_id": character_id,
+                        "character_name": character.character_name,
+                        "success": True,
+                        "request_pk": standing_request.pk,
+                    }
+                )
+                success_count += 1
+
+            except Exception as e:
+                logger.exception(
+                    f"Error creating standing request for character {character_id}: {e}"
+                )
+                results.append(
+                    {
+                        "character_id": character_id,
+                        "success": False,
+                        "error": "An unexpected error occurred.",
+                    }
+                )
+                error_count += 1
+
+        message = f"Requested standings for {success_count} character(s)"
+        if error_count > 0:
+            message += f", {error_count} failed"
+
+        return JsonResponse(
+            {
+                "success": error_count == 0,
+                "message": message,
+                "results": results,
+                "success_count": success_count,
+                "error_count": error_count,
+            }
+        )
+
     except Exception as e:
-        logger.exception(f"Error creating revocation request: {e}")
+        logger.exception(f"Error in bulk character standing request: {e}")
+        return JsonResponse(
+            {"success": False, "error": "An unexpected error occurred."}, status=500
+        )
+
+
+@login_required
+@permission_required("standingsmanager.add_syncedcharacter")
+@require_http_methods(["POST"])
+def api_bulk_remove_standings(request):
+    """
+    API endpoint to create revocation requests for multiple entities.
+
+    Expects JSON body with:
+        entity_ids: list of EVE entity IDs (characters)
+
+    Returns:
+        JSON response with success/error status and details for each entity
+    """
+    try:
+        # Parse JSON body
+        try:
+            data = json.loads(request.body)
+            entity_ids = data.get("entity_ids", [])
+        except json.JSONDecodeError:
+            return JsonResponse(
+                {"success": False, "error": "Invalid JSON body."}, status=400
+            )
+
+        if not entity_ids:
+            return JsonResponse(
+                {"success": False, "error": "No entities specified."}, status=400
+            )
+
+        results = []
+        success_count = 0
+        error_count = 0
+
+        for entity_id in entity_ids:
+            try:
+                # Find entity
+                try:
+                    entity = EveEntity.objects.get(id=entity_id)
+                except EveEntity.DoesNotExist:
+                    results.append(
+                        {
+                            "entity_id": entity_id,
+                            "success": False,
+                            "error": "Entity not found.",
+                        }
+                    )
+                    error_count += 1
+                    continue
+
+                # Check if standing exists
+                if not StandingsEntry.objects.filter(eve_entity=entity).exists():
+                    results.append(
+                        {
+                            "entity_id": entity_id,
+                            "entity_name": entity.name,
+                            "success": False,
+                            "error": "No standing exists for this entity.",
+                        }
+                    )
+                    error_count += 1
+                    continue
+
+                # Check if user owns this entity (for characters)
+                if entity.category == EveEntity.CATEGORY_CHARACTER:
+                    try:
+                        CharacterOwnership.objects.get(
+                            user=request.user, character__character_id=entity_id
+                        )
+                    except CharacterOwnership.DoesNotExist:
+                        results.append(
+                            {
+                                "entity_id": entity_id,
+                                "entity_name": entity.name,
+                                "success": False,
+                                "error": "You can only request removal of your own standings.",
+                            }
+                        )
+                        error_count += 1
+                        continue
+
+                # Create revocation
+                try:
+                    revocation = StandingRevocation.objects.create_for_entity(
+                        entity,
+                        reason=StandingRevocation.RevocationReason.USER_REQUEST,
+                        user=request.user,
+                    )
+                except ValidationError as e:
+                    # Handle Django ValidationError message extraction
+                    if hasattr(e, "messages") and e.messages:
+                        error_msg = str(e.messages[0])
+                    elif hasattr(e, "message"):
+                        error_msg = str(e.message)
+                    else:
+                        error_msg = str(e)
+                    results.append(
+                        {
+                            "entity_id": entity_id,
+                            "entity_name": entity.name,
+                            "success": False,
+                            "error": error_msg,
+                        }
+                    )
+                    error_count += 1
+                    continue
+
+                logger.info(
+                    f"User {request.user.username} created revocation request "
+                    f"for {entity.name}"
+                )
+
+                results.append(
+                    {
+                        "entity_id": entity_id,
+                        "entity_name": entity.name,
+                        "success": True,
+                        "revocation_pk": revocation.pk,
+                    }
+                )
+                success_count += 1
+
+            except Exception as e:
+                logger.exception(
+                    f"Error creating revocation request for entity {entity_id}: {e}"
+                )
+                results.append(
+                    {
+                        "entity_id": entity_id,
+                        "success": False,
+                        "error": "An unexpected error occurred.",
+                    }
+                )
+                error_count += 1
+
+        message = f"Requested revocation for {success_count} entity(ies)"
+        if error_count > 0:
+            message += f", {error_count} failed"
+
+        return JsonResponse(
+            {
+                "success": error_count == 0,
+                "message": message,
+                "results": results,
+                "success_count": success_count,
+                "error_count": error_count,
+            }
+        )
+
+    except Exception as e:
+        logger.exception(f"Error in bulk revocation request: {e}")
         return JsonResponse(
             {"success": False, "error": "An unexpected error occurred."}, status=500
         )
@@ -1088,7 +1475,7 @@ def api_approve_revocation(request, revocation_pk):
         revocation = get_object_or_404(StandingRevocation, pk=revocation_pk)
 
         # Check if still pending
-        if revocation.state != StandingRevocation.RevocationState.PENDING:
+        if revocation.state != StandingRevocation.State.PENDING:
             return JsonResponse(
                 {"success": False, "error": "Revocation is not pending."}, status=400
             )
@@ -1135,7 +1522,7 @@ def api_reject_revocation(request, revocation_pk):
         revocation = get_object_or_404(StandingRevocation, pk=revocation_pk)
 
         # Check if still pending
-        if revocation.state != StandingRevocation.RevocationState.PENDING:
+        if revocation.state != StandingRevocation.State.PENDING:
             return JsonResponse(
                 {"success": False, "error": "Revocation is not pending."}, status=400
             )
